package PPD::Paper;

use PPD;

sub mm($) {
	my ($mm) = @_;
	return int($mm/25.4*72 + 0.5);
}

# Media parameters:
# 	Name		human-readable name
# 	W		width in PS points
# 	H		height in PS points
my %media = (
	'A3' => { Name => 'A3', W => mm(297), H => mm(420) },
	'A4' => { Name => 'A4', W => mm(210), H => mm(297) },
	'A5' => { Name => 'A5', W => mm(148), H => mm(210) },
	'A6' => { Name => 'A6', W => mm(105), H => mm(148) },
	'Letter' => { Name => 'US Letter', W => 612, H => 792 },
	'Legal' => { Name => 'US Legal', W => 612, H => 1008 },
);

# add_papers({
#	MinW => pt,		# Minimum paper width (default: 0)
#	MaxW => pt,		# Maximum paper width (default: 595)
#	MinH => pt,		# Minimum paper height (default: 0)
#	MaxH => pt,		# Maximum paper height (default: 842)
#	MarginH => pt,		# Horizontal margin (default: 0)
#	MarginV => pt,		# Vertical margin (default: 0)
#	DefPaper => paper,	# Default paper type (default: A4)
#	PSPageSize => PS,	# PS code for setting PageSize (subroutine called with ref to media object)
#	PSPageRegion => PS,	# The same for PageRegion (default: use PSPageSize)
# })
sub add_papers($) {
	# Scan available paper formats
	my ($o) = @_;
	my $minw = $o->{MinW} // 0;
	my $maxw = $o->{MaxW} // 595;
	my $minh = $o->{MinH} // 0;
	my $maxh = $o->{MaxH} // 842;
	my $margh = $o->{MarginH} // 0;
	my $margv = $o->{MarginV} // 0;
	my %real_media = ();
	for my $p (keys %media) {
		my $m = $media{$p};
		$m->{W} >= $minw && $m->{W} <= $maxw && $m->{H} >= $minh && $m->{H} <= $maxh or next;
		$real_media{$p} = $m;
		$m->{Key} = $p;
	}
	$o->{DefPaper} //= 'A4';

	define_ui_group('Media', { Name => 'Media settings' });

	option({
		Group => 'Media',
		Key => 'PageSize',
		Name => 'Page Size',
		Choice => 'PickAny',
		Priority => 30,
		Values => [ map {
			my $k = $_;
			my $m = $real_media{$_};
			{
				Key => $k,
				Name => ($m->{Name} // $k),
				PS => &{$o->{PSPageSize}}($m),
				Default => ($k eq $o->{'DefPaper'}),
			}
		} sort keys %real_media ]
	});

	option({
		Group => 'Media',
		Key => 'PageRegion',
		Name => 'Page Region',
		Choice => 'PickAny',
		Priority => 40,
		Values => [ map {
			my $k = $_;
			my $m = $real_media{$_};
			{
				Key => $k,
				Name => ($m->{Name} // $k),
				PS => &{$o->{PSPageRegion} // $o->{PSPageSize}}($m),
				Default => ($k eq $o->{'DefPaper'}),
			}
		} sort keys %real_media ]
	});

	option({
		Group => 'Media',
		Key => 'ImageableArea',
		Values => [ map {
			my $k = $_;
			my $m = $real_media{$_};
			{
				Key => $k,
				Name => ($m->{Name} // $k),
				PS => sprintf("%d %d %d %d", $margh, $margv, $m->{W} - $margh, $m->{H} - $margv),
				Default => ($k eq $o->{'DefPaper'}),
			}
		} sort keys %real_media ]
	});

	option({
		Group => 'Media',
		Key => 'PaperDimension',
		Values => [ map {
			my $k = $_;
			my $m = $real_media{$_};
			{
				Key => $k,
				Name => ($m->{Name} // $k),
				PS => sprintf("%d %d", $m->{W}, $m->{H}),
				Default => ($k eq $o->{'DefPaper'}),
			}
		} sort keys %real_media ]
	});
}

42;
